using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Bezier : MonoBehaviour{

	public Vector3[] points;
	private List<Vector3> control;
	private float[] dist;
	private int segs;

	public void init(List<Vector3> control, int segs){
		this.control = control;
		this.segs = segs;
		this.dist = new float[segs + 1];
		this.points = new Vector3[segs + 1];

		float x = 0, y = 1f / ((float)segs);
		dist[0] = 0;
		points[0] = pointAt(0);

		for (int i = 1; i <= segs; i++){
			x += y;
			points[i] = pointAt(x);
			dist[i] = dist[i - 1] + (points[i] - points[i - 1]).magnitude;
		}
	}

	private Vector3 pointAt(float t){
		List<Vector3> aux = new List<Vector3>(control);
		for (int g = 1; g < control.Count; g++){
			for (int i = 0, j = control.Count - g; i < j; i++){
				aux[i] = (aux[i] * (1 - t)) + (aux[i + 1] * t);
			}
		}
		return aux[0];
	}

	public bool go(GameObject obj, float dist){
		if (dist > this.dist[segs]-20){
			//obj.transform.position = points[segs];
			return false;
		}
		int a = 0, b = segs, mid;

		while (a <= b){
			mid = (a + b) >> 1;
			if (dist < this.dist[mid]) b = mid - 1;
			else a = mid + 1;
		}
		mid = b;
		dist -= this.dist[mid];
		obj.transform.position = points[mid] + ((points[mid + 1] - points[mid]).normalized * dist);
		obj.transform.LookAt(points[mid+1]);
        return true;
	}
}
