using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Connection_GUI : MonoBehaviour{
    public GameObject Luz;
    public GameObject Terrain;
    public GameObject Game_GUI;

    public List<GameObject> Players;
    public List<GameObject> Mines;
    public List<GameObject> Squares;
	public List<GameObject> Beziers;
	public List<GameObject> Bases;
	public List<float> Gold;

    string remoteIP = "G5C17";
    int remotePort = 25000;
    int listenPort = 25000;
    bool useNAT = false;
	public List<float> CreepVelocity;
	public List<float> CreepLife;

    void OnGUI(){
        if (Network.peerType == NetworkPeerType.Disconnected){
            if (GUI.Button(new Rect(10, 10, 100, 30), "Connect")){
                Network.Connect(remoteIP, remotePort);
            }
            if (GUI.Button(new Rect(10, 50, 100, 30), "Start Server")){
				Network.minimumAllocatableViewIDs = 100000;
				Network.InitializeServer(1, listenPort, useNAT);
            }
            remoteIP = GUI.TextField(new Rect(120, 10, 100, 20), remoteIP);
            remotePort = int.Parse(GUI.TextField(new Rect(230, 10, 40, 20), remotePort.ToString()));
        }
        else{
            GUI.Box(new Rect(140, 20, 180, 20), "IP Adress: " + Network.player.ipAddress + ":" + Network.player.port.ToString());
            if (GUI.Button(new Rect(10, 10, 100, 50), "Disconnect")){
                Network.Disconnect(200);
            }
        }
    }

    void OnConnectedToServer(){
        networkView.RPC("init", RPCMode.All);
    }

	void OnPlayerConnected(NetworkPlayer player){
		networkView.SetScope(player, false);
	}

    [RPC]
    void init(){
        Instantiate(Luz);
        Instantiate(Terrain);
		createBeziers();

		if(Network.isServer){
            Instantiate(Players[0], new Vector3(127, 80, 50), Quaternion.Euler(56, 0, 0));
            createSquares(0);
        }
        else{
            Instantiate(Players[1], new Vector3(-95, 80, -128), Quaternion.Euler(56, 0, 0));
            createSquares(1);
        }
        Instantiate(Game_GUI);
    }

    private void createSquares(int id){
		GameObject bezier = GameObject.FindGameObjectWithTag("Bezier"+id);
		Bezier way = (Bezier) bezier.GetComponent("Bezier");

        GameObject temp;
		Mine m;
        for (int i = 50; i < way.points.Length - 50; i += 50){
            Vector3 dir = way.points[i] - way.points[i - 1];
            Vector3 pos = new Vector3(dir.z, 0, -dir.x).normalized;
            Vector3 desl = new Vector3(0, -1f, 0);
            for (int k = 1; k <= 3; k++){
                temp = (GameObject)Network.Instantiate(Squares[id], (way.points[i] + pos * 12 * k) + desl, Quaternion.identity, 0);
                temp.transform.LookAt(way.points[i]);
                networkView.RPC("fixSquare", RPCMode.Others, temp.transform.rotation, id, temp.networkView.viewID);

                temp = (GameObject)Network.Instantiate(Squares[id], (way.points[i] - pos * 12 * k) + desl, Quaternion.identity, 0);
                temp.transform.LookAt(way.points[i]);
                networkView.RPC("fixSquare", RPCMode.Others, temp.transform.rotation, id, temp.networkView.viewID);
            }
        }

        temp = (GameObject)Network.Instantiate(Mines[id], way.points[300], Quaternion.identity, 0);
        temp.transform.LookAt(way.points[300]);
		m = (Mine)temp.GetComponent("Mine");
		m.pos = 0;
		networkView.RPC("fixMine", RPCMode.Others, temp.transform.rotation, id, temp.networkView.viewID, 0);

        temp = (GameObject)Network.Instantiate(Mines[id], way.points[600], Quaternion.identity, 0);
        temp.transform.LookAt(way.points[600]);
		m = (Mine)temp.GetComponent("Mine");
		m.pos = 1;
		networkView.RPC("fixMine", RPCMode.Others, temp.transform.rotation, id, temp.networkView.viewID, 1);

        temp = (GameObject)Network.Instantiate(Mines[id], way.points[800], Quaternion.identity, 0);
        temp.transform.LookAt(way.points[800]);
		m = (Mine)temp.GetComponent("Mine");
		m.pos = 2;
		networkView.RPC("fixMine", RPCMode.Others, temp.transform.rotation, id, temp.networkView.viewID, 2);

		Network.Instantiate(Bases[id], way.points[way.points.Length-1], Quaternion.identity, 0);
    }

    [RPC]
	void fixSquare(Quaternion rot, int id, NetworkViewID idSquare){
        GameObject[] squares = GameObject.FindGameObjectsWithTag("Square" + id);
		foreach (GameObject s in squares){
			if (s.networkView.viewID == idSquare){
				s.transform.rotation = rot;
				return;
			}
		}
    }

    [RPC]
	void fixMine(Quaternion rot, int id, NetworkViewID idMine, int pos){
		GameObject[] mines = GameObject.FindGameObjectsWithTag("Mine" + id);
		foreach (GameObject m in mines){
			if (m.networkView.viewID == idMine){
				m.transform.rotation = rot;
				Mine temp = (Mine)m.GetComponent("Mine");
				temp.pos = pos;
				return;
			}
		}
    }

	[RPC]
	void initCreep(int index, int id, NetworkViewID creepID, int type){
		GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep" + id);
		foreach (GameObject c in creeps){
			Creep creep = (Creep)c.GetComponent("Creep");
			if (c.networkView.viewID == creepID){
				GameObject[] minesEnemy = GameObject.FindGameObjectsWithTag("Mine" + (1 - id));
				if (index < minesEnemy.Length){
					creep.init(minesEnemy[index], id, type);
				}
				else{
					creep.init(null, id, type);
				}
				return;
			}
		}
	}

	[RPC]
	void killBullet(NetworkViewID idBullet){
		GameObject[] bullets = GameObject.FindGameObjectsWithTag("Bullet0");
		foreach (GameObject go in bullets){
			if (go.networkView.viewID == idBullet){
				Destroy(go);
				return;
			}
		}
		bullets = GameObject.FindGameObjectsWithTag("Bullet1");
		foreach (GameObject go in bullets){
			if (go.networkView.viewID == idBullet){
				Destroy(go);
				return;
			}
		}
	}

	[RPC]
	void attackCreep(NetworkViewID idCreep, float attack){
		GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep0");
		foreach (GameObject go in creeps){
			Creep creep = (Creep)go.GetComponent("Creep");
			if (go.networkView.viewID == idCreep){
				if (creep.dano(attack)){
					//Destroy(go);
				}
				return;
			}
		}
		creeps = GameObject.FindGameObjectsWithTag("Creep1");
		foreach (GameObject go in creeps){
			Creep creep = (Creep)go.GetComponent("Creep");
			if (go.networkView.viewID == idCreep){
				if (creep.dano(attack)){
					//Destroy(go);
				}
				return;
			}
		}
	}

	[RPC]
	void initBullet(NetworkViewID idBullet, NetworkViewID idCreep, NetworkViewID idTower){
		GameObject[] bullets = GameObject.FindGameObjectsWithTag("Bullet0");
		foreach (GameObject go in bullets){
			Bullet bullet = (Bullet)go.GetComponent("Bullet");
			if(go.networkView.viewID == idBullet){
				GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep1");
				GameObject[] towers = GameObject.FindGameObjectsWithTag("Tower0");
				GameObject cc = null;
				GameObject tt = null;
				foreach (GameObject creep in creeps){
					if (creep.networkView.viewID == idCreep){
						cc = creep;
						break;
					}
				}
				foreach (GameObject tower in towers){
					if (tower.networkView.viewID == idTower){
						tt = tower;
						break;
					}
				}
				if(cc != null && tt != null) bullet.init(cc, tt);
				return;
			}
		}

		bullets = GameObject.FindGameObjectsWithTag("Bullet1");
		foreach (GameObject go in bullets){
			Bullet bullet = (Bullet)go.GetComponent("Bullet");
			if (go.networkView.viewID == idBullet){
				GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep0");
				GameObject[] towers = GameObject.FindGameObjectsWithTag("Tower1");
				GameObject cc = null;
				GameObject tt = null;
				foreach (GameObject creep in creeps){
					if (creep.networkView.viewID == idCreep){
						cc = creep;
						break;
					}
				}
				foreach (GameObject tower in towers){
					if (tower.networkView.viewID == idTower){
						tt = tower;
						break;
					}
				}
				if (cc != null && tt != null) bullet.init(cc, tt);
				return;
			}
		}
	}

	[RPC]
	void walkCreep(NetworkViewID idCreep, Vector3 position, Quaternion rot){
		GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep0");
		foreach (GameObject go in creeps){
			if (go.networkView.viewID == idCreep){
				go.transform.position = position;
				go.transform.rotation = rot;
				return;
			}
		}
		creeps = GameObject.FindGameObjectsWithTag("Creep1");
		foreach (GameObject go in creeps){
			if (go.networkView.viewID == idCreep){
				go.transform.position = position;
				go.transform.rotation = rot;
				return;
			}
		}
	}
	
	[RPC]
	void danoMine(NetworkViewID idMine, float val, int player){
		Gold[player] += val;
		GameObject[] mines = GameObject.FindGameObjectsWithTag("Mine0");
		foreach (GameObject go in mines){
			Mine mine = (Mine)go.GetComponent("Mine");
			if (go.networkView.viewID == idMine){
				mine.dano(val);
				if (mine.Gold <= 0) Destroy(go);
				return;
			}
		}
		mines = GameObject.FindGameObjectsWithTag("Mine1");
		foreach (GameObject go in mines){
			Mine mine = (Mine)go.GetComponent("Mine");
			if (go.networkView.viewID == idMine){
				mine.dano(val);
				if (mine.Gold <= 0) Destroy(go);
				return;
			}
		}
	}

	[RPC]
	void slowCreep(NetworkViewID idCreep, float slow, float duration){
		GameObject[] creeps = GameObject.FindGameObjectsWithTag("Creep0");
		foreach (GameObject go in creeps){
			Creep creep = (Creep)go.GetComponent("Creep");
			if (go.networkView.viewID == idCreep){
				creep.slow(slow, duration);
				return;
			}
		}
		creeps = GameObject.FindGameObjectsWithTag("Creep1");
		foreach (GameObject go in creeps){
			Creep creep = (Creep)go.GetComponent("Creep");
			if (go.networkView.viewID == idCreep){
				creep.slow(slow, duration);
				return;
			}
		}
	}

	[RPC]
	void killBase(NetworkViewID idBase){
		GameObject b = GameObject.FindGameObjectWithTag("Base0");
		if (b != null && b.networkView.viewID == idBase){
			Destroy(b);
			return;
		}
		b = GameObject.FindGameObjectWithTag("Base1");
		if (b != null && b.networkView.viewID == idBase){
			Destroy(b);
		}
	}

	[RPC]
	void danoBase(NetworkViewID idBase, float attack){
		GameObject b = GameObject.FindGameObjectWithTag("Base0");
		if (b != null && b.networkView.viewID == idBase){
			Base bs = (Base)b.GetComponent("Base");
			bs.dano(attack);
			return;
		}
		b = GameObject.FindGameObjectWithTag("Base1");
		if (b != null && b.networkView.viewID == idBase){
			Base bs = (Base)b.GetComponent("Base");
			bs.dano(attack);
		}
	}

	void createBeziers(){
		GameObject b0 = (GameObject) Instantiate(Beziers[0]);
		GameObject b1 = (GameObject) Instantiate(Beziers[1]);
		Bezier bezier0 = (Bezier)b0.GetComponent("Bezier");
		Bezier bezier1 = (Bezier)b1.GetComponent("Bezier");

		List<Vector3> lista0 = new List<Vector3>();
		lista0.Add(new Vector3(-141, 1, -105));
		lista0.Add(new Vector3(-195, 1, 117));
		lista0.Add(new Vector3(82, 1, 144));
		List<Vector3> lista1 = new List<Vector3>();
		lista1.Add(new Vector3(134, 1, 112));
		lista1.Add(new Vector3(187, 1, -134));
		lista1.Add(new Vector3(-89, 1, -117));

		bezier0.init(lista0, 1000);
		bezier1.init(lista1, 1000);
	}
}
